// Copyright (C) 1999-2000 Id Software, Inc.
//
#include "ui_local.h"



#define ID_ADD	 100
#define ID_MAINMENU	 101

typedef struct
{
	menuframework_s	menu;
	menubitmap_s	mainmenu;
	menubitmap_s	add;
	char			info[MAX_INFO_STRING];
	int				numlines;
	qhandle_t		cornerLower;
	
} serverinfo_t;

static serverinfo_t	s_serverinfo;


/*
=================
Favorites_Add

Add current server to favorites
=================
*/
void Favorites_Add( void )
{
	char	adrstr[128];
	char	serverbuff[128];
	int		i;
	int		best;

	trap_Cvar_VariableStringBuffer( "cl_currentServerAddress", serverbuff, sizeof(serverbuff) );
	if (!serverbuff[0])
		return;

	best = 0;
	for (i=0; i<MAX_FAVORITESERVERS; i++)
	{
		trap_Cvar_VariableStringBuffer( va("server%d",i+1), adrstr, sizeof(adrstr) );
		if (!Q_stricmp(serverbuff,adrstr))
		{
			// already in list
			return;
		}
		
		// use first empty or non-numeric available slot
		if ((adrstr[0]  < '0' || adrstr[0] > '9' ) && !best)
			best = i+1;
	}

	if (best)
		trap_Cvar_Set( va("server%d",best), serverbuff);
}


/*
=================
ServerInfo_Event
=================
*/
static void ServerInfo_Event( void* ptr, int event )
{
	switch (((menucommon_s*)ptr)->id)
	{
		case ID_ADD:
			if (event != QM_ACTIVATED)
				break;
		
			Favorites_Add();
			UI_PopMenu();
			break;

		case ID_MAINMENU:
			if (event != QM_ACTIVATED)
				break;

			UI_PopMenu();
			break;
	}
}

/*
=================
ServerInfoMenu_Graphics
=================
*/
void ServerInfoMenu_Graphics (void)
{
	// Draw the basic screen layout
	UI_MenuFrame2(&s_serverinfo.menu);

	trap_R_SetColor( colorTable[CT_LTPURPLE1]);
	UI_DrawHandlePic(30,202,  47, 188, uis.whiteShader);
}

/*
=================
ServerInfo_MenuDraw
=================
*/
static void ServerInfo_MenuDraw( void )
{
	const char		*s;
	char			key[MAX_INFO_KEY];
	char			value[MAX_INFO_VALUE];
	int				y;

	ServerInfoMenu_Graphics();

	y = 62;
	s = s_serverinfo.info;
	while ( s ) 
	{
		Info_NextPair( &s, key, value );
		if ( !key[0] ) 
		{
			break;
		}

		Q_strcat( key, MAX_INFO_KEY, ":" ); 

		UI_DrawProportionalString(SCREEN_WIDTH*0.50 - 8,y,key,UI_RIGHT|UI_SMALLFONT,colorTable[CT_LTBLUE2]);
		UI_DrawProportionalString(SCREEN_WIDTH*0.50 + 8,y,value,UI_LEFT|UI_SMALLFONT,colorTable[CT_LTGOLD1]);

		y += SMALLCHAR_HEIGHT;
	}

	Menu_Draw( &s_serverinfo.menu );
}

/*
=================
ServerInfo_MenuKey
=================
*/
static sfxHandle_t ServerInfo_MenuKey( int key )
{
	return ( Menu_DefaultKey( &s_serverinfo.menu, key ) );
}

/*
=================
ServerInfo_Cache
=================
*/
void ServerInfo_Cache( void )
{
	s_serverinfo.cornerLower = trap_R_RegisterShaderNoMip("menu/common/corner_ll_47_18.tga");

}

/*
=================
PlayerSettings_MenuInit
=================
*/
static void UI_ServerInfoMenu_Init(void) 
{
	const char		*s;
	char			key[MAX_INFO_KEY];
	char			value[MAX_INFO_VALUE];

	// zero set all our globals
	memset( &s_serverinfo, 0 ,sizeof(serverinfo_t) );

	ServerInfo_Cache();

	s_serverinfo.menu.draw						= ServerInfo_MenuDraw;
	s_serverinfo.menu.key						= ServerInfo_MenuKey;
	s_serverinfo.menu.wrapAround				= qtrue;
	s_serverinfo.menu.fullscreen				= qtrue;
	s_serverinfo.menu.nitems					= 0;
	s_serverinfo.menu.descX						= MENU_DESC_X;
	s_serverinfo.menu.descY						= MENU_DESC_Y;
	s_serverinfo.menu.listX						= 230;
	s_serverinfo.menu.listY						= 188;
	s_serverinfo.menu.titleX					= MENU_TITLE_X;
	s_serverinfo.menu.titleY					= MENU_TITLE_Y;
	s_serverinfo.menu.titleI					= MNT_SERVERINFO_TITLE;
	s_serverinfo.menu.footNoteEnum				= MNT_SERVERINFO;

	// Button Data
	s_serverinfo.mainmenu.generic.type			= MTYPE_BITMAP;      
	s_serverinfo.mainmenu.generic.flags			= QMF_HIGHLIGHT_IF_FOCUS;
	s_serverinfo.mainmenu.generic.x				= 112;
	s_serverinfo.mainmenu.generic.y				= 400;
	s_serverinfo.mainmenu.generic.name			= "menu/common/square.tga";
	s_serverinfo.mainmenu.generic.id				= ID_MAINMENU;
	s_serverinfo.mainmenu.generic.callback		= ServerInfo_Event;
	s_serverinfo.mainmenu.width					= MENU_BUTTON_MED_WIDTH;
	s_serverinfo.mainmenu.height					= MENU_BUTTON_MED_HEIGHT;
	s_serverinfo.mainmenu.color					= CT_DKPURPLE1;
	s_serverinfo.mainmenu.color2					= CT_LTPURPLE1;
	s_serverinfo.mainmenu.textX					= MENU_BUTTON_TEXT_X;
	s_serverinfo.mainmenu.textY					= MENU_BUTTON_TEXT_Y;
	s_serverinfo.mainmenu.textEnum				= MBT_INGAMEMENU;
	s_serverinfo.mainmenu.textcolor				= CT_BLACK;
	s_serverinfo.mainmenu.textcolor2				= CT_WHITE;

	s_serverinfo.add.generic.type		= MTYPE_BITMAP;      
	s_serverinfo.add.generic.flags		= QMF_HIGHLIGHT_IF_FOCUS;
	s_serverinfo.add.generic.x			= 446;
	s_serverinfo.add.generic.y			= 400;
	s_serverinfo.add.generic.name		= "menu/common/square.tga";
	s_serverinfo.add.generic.id			= ID_ADD;
	s_serverinfo.add.generic.callback	= ServerInfo_Event;
	s_serverinfo.add.width				= MENU_BUTTON_MED_WIDTH;
	s_serverinfo.add.height				= MENU_BUTTON_MED_HEIGHT;
	s_serverinfo.add.color				= CT_DKPURPLE1;
	s_serverinfo.add.color2				= CT_LTPURPLE1;
	s_serverinfo.add.textX				= MENU_BUTTON_TEXT_X;
	s_serverinfo.add.textY				= MENU_BUTTON_TEXT_Y;
	s_serverinfo.add.textEnum			= MBT_ADDTOFAVS;
	s_serverinfo.add.textcolor			= CT_BLACK;
	s_serverinfo.add.textcolor2			= CT_WHITE;
	

	if( trap_Cvar_VariableValue( "sv_running" ) ) 
	{
		s_serverinfo.add.generic.flags |= QMF_GRAYED;
	}

	trap_GetConfigString( CS_SERVERINFO, s_serverinfo.info, MAX_INFO_STRING );

	s_serverinfo.numlines = 0;
	s = s_serverinfo.info;
	while ( s ) 
	{
		Info_NextPair( &s, key, value );
		if ( !key[0] ) 
		{
			break;
		}
		s_serverinfo.numlines++;
	}

	if (s_serverinfo.numlines > 16)
	{
		s_serverinfo.numlines = 16;
	}

	Menu_AddItem( &s_serverinfo.menu, (void*) &s_serverinfo.mainmenu );
	Menu_AddItem( &s_serverinfo.menu, (void*) &s_serverinfo.add );

}

/*
=================
UI_ServerInfoMenu
=================
*/
void UI_ServerInfoMenu( void )
{

	UI_ServerInfoMenu_Init();

	UI_PushMenu( &s_serverinfo.menu );
}


